/*=============================================================================
XMOTO
Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)

This file is part of XMOTO.

XMOTO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

XMOTO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with XMOTO; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

/* 
 *  GUI: list control
 */
#include "VXml.h"
#include "GUI.h"

namespace vapp {

  /*===========================================================================
  Painting
  ===========================================================================*/
  void UIList::paint(void) {
    bool bDisabled = isDisabled();
  
    /* Draw list frame */
    putElem(0,0,-1,-1,UI_ELEM_FRAME_TL,false);
    putElem(getPosition().nWidth-8,0,-1,-1,UI_ELEM_FRAME_TR,false);
    putElem(getPosition().nWidth-8,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BR,false);
    putElem(0,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BL,false);
    putElem(8,0,getPosition().nWidth-16,-1,UI_ELEM_FRAME_TM,false);
    putElem(8,getPosition().nHeight-8,getPosition().nWidth-16,-1,UI_ELEM_FRAME_BM,false);
    putElem(0,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_ML,false);
    putElem(getPosition().nWidth-8,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_MR,false);
    putRect(8,8,getPosition().nWidth-16,getPosition().nHeight-16,MAKE_COLOR(0,0,0,127));
    
    /* Draw column headers */
    int nHeaderHeight = 18;
    int nHX = 6,nHY = 6;
    setTextSolidColor(MAKE_COLOR(188,186,67,255));
    for(int i=0;i<m_Columns.size();i++) {
      putText(nHX,nHY + (nHeaderHeight*2)/3,m_Columns[i]);
      nHX += m_ColumnWidths[i];            
    }
    putRect(6,nHeaderHeight+6,getPosition().nWidth-12,2,MAKE_COLOR(188,186,67,255));
    
    /* Render list */
    int nLX = 6, nLY = nHeaderHeight+6+4;
    int nLWidth = getPosition().nWidth-12;
    int nLHeight = getPosition().nHeight - nLY - 6;
    int nRowHeight = 16;
                
    setTextSolidColor(MAKE_COLOR(255,255,255,255));
    
    for(int i=0;i<m_Entries.size();i++) {
      int y = i*nRowHeight + (nRowHeight*2)/3;
      
      if(m_nSelected == i) {
        Color c = MAKE_COLOR(70,70,70,255);
        if(!bDisabled) c = MAKE_COLOR(160,40,40,255);
        putRect(nLX,nLY+i*nRowHeight,nLWidth,nRowHeight,c);
      }              
      
      int x = 0;      
      for(int j=0;j<m_Entries[i]->Text.size();j++) {              
        putText(nLX+x,nLY+y,m_Entries[i]->Text[j]);
        x += m_ColumnWidths[j];                
      }
    }
  }

  /*===========================================================================
  Mouse event handling
  ===========================================================================*/
  void UIList::mouseLDown(int x,int y) {
    /* Find out what item is affected */
    int nHeaderHeight=18;
    int nLX = 6, nLY = nHeaderHeight+6+4;
    int nLWidth = getPosition().nWidth-12;
    int nLHeight = getPosition().nHeight - nLY - 6;
    int nRowHeight = 16;    
    for(int i=0;i<m_Entries.size();i++) {
      int yy = nLY + i*nRowHeight;
      if(x >= nLX && x <getPosition().nWidth-6 &&
         y >= yy && y < yy+nRowHeight) {
        /* Select this */
        m_nSelected = i;
        break;
      }
    }
  }
  
  void UIList::mouseLUp(int x,int y) {
  }
  
  void UIList::mouseRDown(int x,int y) {
  }
  
  void UIList::mouseRUp(int x,int y) {
  }

  void UIList::mouseHover(int x,int y) {
  }

  /*===========================================================================
  Allocate entry / vice versa
  ===========================================================================*/
  UIListEntry *UIList::addEntry(std::string Text,void *pvUser) {
    UIListEntry *p = new UIListEntry;
    p->Text.push_back(Text);
    p->pvUser = pvUser;
    m_Entries.push_back(p);
    return p;
  }
  
  void UIList::clear(void) {
    _FreeUIList();
    m_Entries.clear();
  }

  /*===========================================================================
  Free entries
  ===========================================================================*/
  void UIList::_FreeUIList(void) {
    for(int i=0;i<m_Entries.size();i++)
      delete m_Entries[i];
  }

};

