/*=============================================================================
XMOTO
Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)

This file is part of XMOTO.

XMOTO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

XMOTO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with XMOTO; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

/* 
 *  XML support through the tinyxml library.
 */

#include "VXml.h"
#include "VFileIO.h"

namespace vapp {

  /*===========================================================================
  Load XML document from disk
  ===========================================================================*/
  void XMLDocument::readFromFile(std::string File) {
    /* Clean up if a doc already is loaded */
    if(m_pXML) delete m_pXML;
    
    /* Load */
    std::string Line,Doc = "";    
    
    FileHandle *pfh = FS::openIFile(File);
    if(pfh==NULL) return;

    m_pXML = new TiXmlDocument;
    
    while(FS::readNextLine(pfh,Line)) {
      if(Line.length() > 0) {
        Doc.append(Line);
        Doc.append("\n");
      }
    }    
    FS::closeFile(pfh);

    m_pXML->Parse(Doc.c_str());
    if(m_pXML->Error()) {
      /* TODO: output error */
      //logPrintF("*** XML error *** : %s : %s",Path.c_str(),pXML->ErrorDesc());
    }
  }
      
  /*===========================================================================
  Save XML document from disk
  ===========================================================================*/
  void XMLDocument::writeToFile(std::string File) {
    /* Anything? */
    if(!m_pXML) return;
    FileHandle *pfh = FS::openOFile(File);
    if(pfh==NULL || pfh->Type!=FHT_STDIO) return;
    m_pXML->Print(pfh->fp);
    FS::closeFile(pfh);
  }
    
};

